/*
 * Copyright (c) 1998-2008 TeamDev Ltd. All Rights Reserved.
 * Use is subject to license terms.
 */
package com.teamdev.xpcom.mozilla.demo.components;

import static com.teamdev.xpcom.mozilla.demo.JxBrowserDemo._aboutBlank;

import java.awt.BorderLayout;
import java.awt.Component;
import java.awt.Dimension;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.FocusAdapter;
import java.awt.event.FocusEvent;
import java.awt.event.InputEvent;
import java.awt.event.KeyEvent;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import java.lang.reflect.Method;
import java.net.MalformedURLException;
import java.net.URL;
import java.text.MessageFormat;
import java.util.HashMap;
import java.util.Map;

import javax.swing.AbstractAction;
import javax.swing.ComboBoxModel;
import javax.swing.Icon;
import javax.swing.ImageIcon;
import javax.swing.JComboBox;
import javax.swing.JLabel;
import javax.swing.JList;
import javax.swing.JPanel;
import javax.swing.JRootPane;
import javax.swing.JTextField;
import javax.swing.KeyStroke;
import javax.swing.ListCellRenderer;
import javax.swing.SwingUtilities;
import javax.swing.border.EmptyBorder;

import com.teamdev.xpcom.mozilla.demo.LocationListener;
import com.teamdev.xpcom.mozilla.demo.components.icon.IconBuilder;
import com.teamdev.xpcom.mozilla.demo.components.icon.IconLoader;

/*
 * LocationCombo - the GUI Location Box of web browser
 */
public class LocationCombo extends JComboBox
{

    private static final long serialVersionUID = -8346148454465688521L;

    private static final Icon DEFAULT_ICON = new ImageIcon(LocationCombo.class
            .getResource("res/doc.gif"));

    private static final String HTTP_PROTOCOL_ID = "http://";

    private static final String DEFAULT_URL_PATTERN = HTTP_PROTOCOL_ID
            + "www.{0}.com";

    private static final String HTTP_URL_PATTERN = "http://" + "{0}" + "/";

    private Map<String, Icon> hostIcons = new HashMap<String, Icon>();

    private MyEditor myEditor = new MyEditor();

    public void addLocationListener(LocationListener locationListener)
    {
        listenerList.add(LocationListener.class, locationListener);
    }

    public void removeLocationListener(LocationListener locationListener)
    {
        listenerList.remove(LocationListener.class, locationListener);
    }

    private void setLocation(String url, String parentComponent)
    {
        if (parentComponent.equals("TEXT"))
        {
            LocationListener[] listeners = listenerList
                    .getListeners(LocationListener.class);
            for (LocationListener locationListener : listeners)
            {
                locationListener.browserNavigate(url);
                addUrlToComboBox(url);
            }
        }
        else if (parentComponent.equals("COMBO"))
        {
            LocationListener[] listeners = listenerList
                    .getListeners(LocationListener.class);
            for (LocationListener locationListener : listeners)
            {
                locationListener.browserNavigate(url);
            }
        }
        myEditor.loadIcon();
    }

    private void addUrlToComboBox(String url)
    {
        String newUrl;
        if (url.startsWith(HTTP_PROTOCOL_ID))
        {
            newUrl = url;
        }
        else
        {
            if (!url.startsWith("http://") && !url.startsWith("https://")
                    && !url.startsWith("ftp://"))
            {
                newUrl = MessageFormat.format(HTTP_URL_PATTERN, url);
            }
            else
            {
                newUrl = url;
            }
        }
        ComboBoxModel comboBoxModel = this.getModel();
        boolean existUrl = false;
        for (int i = 0; i < comboBoxModel.getSize(); i++)
        {
            String elementComboBox = (String) comboBoxModel.getElementAt(i);
            if (elementComboBox.equals(newUrl) || elementComboBox.equals(url))
            {
                existUrl = true;
                break;
            }
        }
        if (!existUrl)
        {
            this.insertItemAt(newUrl, 0);
            this.setSelectedIndex(0);
        }
    }

    public LocationCombo()
    {
        super();
        setRenderer(new MyCellSender());
        setEditor(myEditor);
        setEditable(true);

        myEditor.addActionListener(new ActionListener()
        {
            public void actionPerformed(ActionEvent event)
            {
                setLocation(myEditor.getInputBox().getText(), "TEXT");
            }
        });

        this.addActionListener(new ActionListener()
        {
            public void actionPerformed(ActionEvent event)
            {
                if (event.getActionCommand().equals("comboBoxChanged")
                        && event.getModifiers() > 0)
                {
                    String selectedItem = (String) LocationCombo.this
                            .getSelectedItem();
                    if (selectedItem != null && !selectedItem.equals(""))
                    {
                        setLocation((String) LocationCombo.this
                                .getSelectedItem(), "COMBO");
                    }
                }
            }
        });

    }

    public MyEditor getMyEditor()
    {
        return myEditor;
    }

    public void setURLText(String uri)
    {
        myEditor.setText(uri);
    }

    private static String getHost(String url)
    {
        String result = "";
        try
        {
            result = new URL(url).getHost();
        }
        catch (MalformedURLException e)
        {
        }
        return result;
    }

    public void requestFocus()
    {
        myEditor.requestFocus();
    }

    @SuppressWarnings("serial")
    public class MyEditor extends javax.swing.JPanel implements
            javax.swing.ComboBoxEditor
    {
        public static final int STATE_END_LOADING = 0;

        public static final int STATE_FOCUS_GAINED = 1;

        public static final int STATE_FOCUS_LOST = 2;

        public static final int STATE_TAB_CHANGED = 3;

        private Object oldValue;

        private JTextField inputBox;

        private JLabel icon;

        private LocationPopupMenu popupMenu;

        private IconBuilder iconBuilder = IconBuilder.createBuilder();

        public MyEditor()
        {
            super();
            inputBox = new JTextField();
            icon = new JLabel();
            init();
        }

        private void init()
        {
            setBorder(null);
            setLayout(new BorderLayout());
            setBackground(inputBox.getBackground());
            icon = new JLabel(DEFAULT_ICON);
            icon.setPreferredSize(new Dimension(20, 20));
            inputBox.setBorder(null);

            String ctrlDownKey = "CTRL_ENTER";
            inputBox.getInputMap().put(
                    KeyStroke.getKeyStroke(KeyEvent.VK_ENTER,
                            InputEvent.CTRL_MASK), "CTRL_ENTER");
            inputBox.getActionMap().put(ctrlDownKey, new AbstractAction()
            {
                private static final long serialVersionUID = -2314835413235590418L;

                public void actionPerformed(ActionEvent e)
                {
                    final String text = inputBox.getText();
                    String newURL = null;
                    if (!text.startsWith("http://")
                            && !text.startsWith("https://")
                            && !text.startsWith("ftp://"))
                    {
                        newURL = MessageFormat
                                .format(DEFAULT_URL_PATTERN, text);
                    }
                    else
                    {
                        newURL = text;
                    }

                    inputBox.setText(newURL);
                    inputBox.postActionEvent();
                }
            });
            add(icon, BorderLayout.WEST);
            add(inputBox, BorderLayout.CENTER);
            popupMenu = new LocationPopupMenu(inputBox);
            inputBox.setComponentPopupMenu(popupMenu.getMenu());
            inputBox.addMouseListener(new MouseAdapter()
            {
                private boolean firstClicked;

                public void mouseEntered(MouseEvent e)
                {
                    firstClicked = !inputBox.isFocusOwner();
                }

                public void mousePressed(MouseEvent e)
                {
                    // right button pressed (BUTTON3 pressed)
                    if ((e.getModifiers() & MouseEvent.BUTTON1_MASK) == 0)
                    {
                        if ((e.getModifiers() & MouseEvent.BUTTON1_MASK) == 0)
                        {
                            String selectedText = inputBox.getSelectedText();
                            if (selectedText == null && firstClicked)
                            {
                                inputBox.selectAll();
                            }
                            firstClicked = false;
                            popupMenu.createMenu();
                        }
                    }
                }

                public void mouseReleased(MouseEvent e)
                {
                    // left button released (BUTTON1 released)
                    if ((e.getModifiers() & MouseEvent.BUTTON3_MASK) == 0)
                    {
                        String selectedText = inputBox.getSelectedText();
                        if (selectedText == null && firstClicked)
                        {
                            inputBox.selectAll();
                        }
                        firstClicked = false;
                    }
                }
            });

            inputBox.addFocusListener(new FocusAdapter()
            {
                public void focusLost(FocusEvent event)
                {
                    if (!(event.isTemporary() && event.getOppositeComponent() instanceof JRootPane))
                    {
                        setFocusAndSelectedText(STATE_FOCUS_LOST);
                    }
                }
            });
        }

        public void setFocusAndSelectedText(final int state)
        {
            if (state == STATE_FOCUS_LOST)
            {
                if (inputBox.getSelectedText() != null)
                {
                    inputBox.select(0, 0);
                }
            }
            else if (state == STATE_TAB_CHANGED)
            {
                SwingUtilities.invokeLater(new Runnable()
                {
                    public void run()
                    {
                        if (inputBox.getText().equals(_aboutBlank)
                                && inputBox.isFocusOwner())
                        {
                            inputBox.selectAll();
                            inputBox.requestFocusInWindow();
                        }
                    }
                });
            }
            else if (state == STATE_END_LOADING)
            {
                SwingUtilities.invokeLater(new Runnable()
                {
                    public void run()
                    {
                        inputBox.selectAll();
                        inputBox.requestFocusInWindow();
                    }
                });
            }
        }

        public JTextField getInputBox()
        {
            return inputBox;
        }

        void loadIcon()
        {
            try
            {
                URL inputURL = new URL(inputBox.getText());
                String hostIcon = inputURL.getHost();

                if (hostIcons.containsKey(hostIcon))
                {
                    icon.setIcon((Icon) hostIcons.get(hostIcon));
                }
                else
                {
                    try
                    {
                        Icon newIcon = loadIcon(inputURL.toString());
                        newIcon = newIcon != null ? newIcon : DEFAULT_ICON;
                        icon.setIcon(newIcon);
                        hostIcons.put(hostIcon, newIcon);
                    }
                    catch (Exception e)
                    {
                        icon.setIcon(DEFAULT_ICON);
                        hostIcons.put(hostIcon, DEFAULT_ICON);
                    }
                }
                icon.repaint();
            }
            catch (MalformedURLException e)
            {
                icon.setIcon(DEFAULT_ICON);
                icon.repaint();
            }
        }

        void setText(String text)
        {
            inputBox.setText(text);
            loadIcon();
        }

        private Icon loadIcon(String url) throws Exception
        {
            IconLoader iconLoader = iconBuilder.newIconLoader();
            return iconLoader.loadFromURL(url);
        }

        public void requestFocus()
        {
            inputBox.requestFocusInWindow();
        }

        public void addActionListener(ActionListener l)
        {
            inputBox.addActionListener(l);
        }

        public Component getEditorComponent()
        {
            return this;
        }

        @SuppressWarnings("unchecked")
        public Object getItem()
        {
            Object newValue = inputBox.getText();
            if (oldValue != null && !(oldValue instanceof String))
            {
                if (newValue.equals(oldValue.toString()))
                {
                    return oldValue;
                }
                else
                {
                    Class cls = oldValue.getClass();
                    try
                    {
                        Method method = cls.getMethod("valueOf",
                                new Class[] { String.class });
                        newValue = method.invoke(oldValue, inputBox.getText());
                    }
                    catch (Exception ex)
                    {
                    }
                }
            }
            return newValue;
        }

        public void removeActionListener(ActionListener l)
        {
            inputBox.removeActionListener(l);
        }

        public void selectAll()
        {
            inputBox.selectAll();
        }

        public void setItem(Object anObject)
        {
            if (anObject != null)
            {
                inputBox.setText(anObject.toString());
                oldValue = anObject;
            }
            else
            {
                inputBox.setText(_aboutBlank);
            }
        }

    }

    private class MyCellSender extends JPanel implements ListCellRenderer
    {

        private static final long serialVersionUID = -2274314133433245087L;

        JLabel icon;

        MyCellSender()
        {
            super();
            setBorder(null);
            icon = new JLabel();
            icon.setBorder(new EmptyBorder(1, 4, 1, 0));
            setLayout(new BorderLayout());
            add(icon, BorderLayout.WEST);
        }

        public Component getListCellRendererComponent(JList list, Object value,
                int index, boolean isSelected, boolean cellHasFocus)
        {
            String text = value.toString();
            icon.setText(text);
            if (index != -1)
            {
                icon.setIcon((Icon) hostIcons.get(getHost(text)));
                if (isSelected)
                {
                    icon.setForeground(list.getSelectionForeground());
                    setBackground(list.getSelectionBackground());
                }
                else
                {
                    icon.setForeground(list.getForeground());
                    setBackground(list.getBackground());
                }
            }
            return this;
        }
    }

}
