/*
 * Copyright (c) 1998-2008 TeamDev Ltd. All Rights Reserved.
 * Use is subject to license terms.
 */
package com.teamdev.xpcom.mozilla.demo.components;

import java.awt.KeyboardFocusManager;
import java.awt.Toolkit;
import java.awt.datatransfer.Clipboard;
import java.awt.datatransfer.DataFlavor;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;

import javax.swing.AbstractAction;
import javax.swing.Action;
import javax.swing.JComponent;
import javax.swing.JMenuItem;
import javax.swing.JPopupMenu;
import javax.swing.JTextField;
import javax.swing.TransferHandler;

public class LocationPopupMenu
{
    private JPopupMenu _popupMenu;

    private JTextField _jComponent;

    private JMenuItem _cut;

    private JMenuItem _copy;

    private JMenuItem _paste;

    private JMenuItem _delete;

    public JPopupMenu getMenu()
    {
        return _popupMenu;
    }

    public LocationPopupMenu(JTextField parentComponent)
    {
        _jComponent = parentComponent;
        TransferActionListener actionListener = new TransferActionListener();

        _cut = new JMenuItem("Cut");
        _cut.setActionCommand((String) TransferHandler.getCutAction().getValue(Action.NAME));
        _cut.addActionListener(actionListener);

        _copy = new JMenuItem("Copy");
        _copy.setActionCommand((String) TransferHandler.getCopyAction().getValue(Action.NAME));
        _copy.addActionListener(actionListener);

        _paste = new JMenuItem("Paste");
        _paste.setActionCommand((String) TransferHandler.getPasteAction().getValue(Action.NAME));
        _paste.addActionListener(actionListener);

        _delete = new JMenuItem("Delete");
        _delete.addActionListener(new DeleteAction());

        _popupMenu = new JPopupMenu();
        _popupMenu.add(_cut);
        _popupMenu.add(_copy);
        _popupMenu.add(_paste);
        _popupMenu.add(_delete);
    }

    public void createMenu()
    {
        String selectedText = _jComponent.getSelectedText();
        if (selectedText != null && !selectedText.equals(""))
        {
            _cut.setEnabled(true);
            _copy.setEnabled(true);
            _delete.setEnabled(true);
        }
        else
        {
            _cut.setEnabled(false);
            _copy.setEnabled(false);
            _delete.setEnabled(false);
        }

        Clipboard clipboard = Toolkit.getDefaultToolkit().getSystemClipboard();
        DataFlavor flavor = DataFlavor.stringFlavor;
        if (clipboard.isDataFlavorAvailable(flavor))
        {
            _paste.setEnabled(true);
        }
        else
        {
            _paste.setEnabled(false);
        }
    }

    private class DeleteAction extends AbstractAction
    {
        public void actionPerformed(ActionEvent e)
        {
            String allText = _jComponent.getText();
            String selectedText = _jComponent.getSelectedText();
            if (selectedText != null)
            {
                String text = allText.substring(0, allText.indexOf(selectedText))
                        + allText.substring(allText.indexOf(selectedText) + selectedText.length(), allText.length());
                _jComponent.setText(text);
            }
        }
    }

    private class TransferActionListener implements ActionListener, PropertyChangeListener
    {
        private JComponent focusOwner = null;

        public TransferActionListener()
        {
            KeyboardFocusManager manager = KeyboardFocusManager.getCurrentKeyboardFocusManager();
            manager.addPropertyChangeListener("permanentFocusOwner", this);
        }

        public void propertyChange(PropertyChangeEvent event)
        {
            Object object = event.getNewValue();
            if (object instanceof JComponent)
            {
                focusOwner = (JComponent) object;
            }
            else
            {
                focusOwner = null;
            }
        }

        public void actionPerformed(ActionEvent event)
        {
            if (focusOwner == null) { return; }
            Action action = focusOwner.getActionMap().get(event.getActionCommand());
            if (action != null)
            {
                action.actionPerformed(new ActionEvent(focusOwner, ActionEvent.ACTION_PERFORMED, null));
            }
        }
    }

}
