/*
 * Copyright (c) 1998-2008 TeamDev Ltd. All Rights Reserved.
 * Use is subject to license terms.
 */
package com.teamdev.xpcom.mozilla.demo.components;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Component;
import java.awt.Dimension;
import java.awt.FlowLayout;
import java.awt.Frame;
import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Insets;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.ItemEvent;
import java.awt.event.ItemListener;
import java.awt.event.KeyEvent;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;
import java.net.MalformedURLException;
import java.net.URL;
import java.util.prefs.Preferences;

import javax.swing.BoxLayout;
import javax.swing.ButtonGroup;
import javax.swing.InputVerifier;
import javax.swing.JButton;
import javax.swing.JCheckBox;
import javax.swing.JComponent;
import javax.swing.JDialog;
import javax.swing.JFormattedTextField;
import javax.swing.JLabel;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JRadioButton;
import javax.swing.JTextField;
import javax.swing.KeyStroke;
import javax.swing.border.Border;
import javax.swing.border.CompoundBorder;
import javax.swing.border.EmptyBorder;
import javax.swing.event.DocumentEvent;
import javax.swing.event.DocumentListener;
import javax.swing.text.AttributeSet;
import javax.swing.text.BadLocationException;
import javax.swing.text.DocumentFilter;
import javax.swing.text.InternationalFormatter;

import org.jdesktop.layout.GroupLayout;

import com.teamdev.jxbrowser.BrowserAttributes;
import com.teamdev.xpcom.ProxyConfiguration;
import com.teamdev.xpcom.Services;
import com.teamdev.xpcom.Xpcom;

public class OptionsDialog extends JDialog
{
    private static final long serialVersionUID = 1088003199678715423L;

    private JPanel _contentPanel;

    private JPanel _connectionSettingsPanel;

    private JButton _buttonOK;

    private JButton _buttonCancel;

    private JCheckBox _loadImagesChBox;

    private JCheckBox _enableJavascriptChBox;

    private JCheckBox _enablePluginsChBox;

    private JCheckBox _blockSubframesChBox;

    private JCheckBox _blockMetaredirectsChBox;

    private JRadioButton _directConnectionRadioButton;

    private JRadioButton _webProxyAutoDiscoveryRadioButton;

    private JRadioButton _manualProxyConfigurationRadioButton;

    private JRadioButton _automaticProxyConfigurationURLRadioButton;

    private JTextField _httpProxyTextField;

    private PortTextField _httpPortTextField;

    private JTextField _sslProxyTextField;

    private PortTextField _sslPortTextField;

    private JTextField _ftpProxyTextField;

    private PortTextField _ftpPortTextField;

    private JCheckBox _useThisProxyServerCheckBox;

    private JTextField _gopherProxyTextField;

    private PortTextField _gopherPortTextField;

    private JTextField _socksProxyTextField;

    private PortTextField _socksPortTextField;

    private JRadioButton _socksV4RadioButton;

    private JRadioButton _socksV5RadioButton;

    private JTextField _automaticProxyConfigurationURLTextField;

    private JTextField _noProxyTextField;

    private JPanel _manualProxyConfigurationPanel;

    private final static Preferences _userBrowserContent = Preferences.userNodeForPackage(OptionsDialog.class);
    private final static String ALLOW_IMAGES = "ALLOW_IMAGES";
    private final static String ALLOW_JAVASCRIPT = "ALLOW_JAVASCRIPT";
    private final static String ALLOW_METAREDIRECTS = "ALLOW_METAREDIRECTS";
    private final static String ALLOW_PLUGINS = "ALLOW_PLUGINS";
    private final static String ALLOW_SUBFRAMES = "ALLOW_SUBFRAMES";
    public final static BrowserAttributes DOC_ATTR;
    static
    {
        DOC_ATTR = new BrowserAttributes();
        DOC_ATTR.setAllowImages(Boolean.parseBoolean(_userBrowserContent.get(ALLOW_IMAGES, "true")));
        DOC_ATTR.setAllowJavascript(Boolean.parseBoolean(_userBrowserContent.get(ALLOW_JAVASCRIPT, "true")));
        DOC_ATTR.setAllowMetaRedirects(Boolean.parseBoolean(_userBrowserContent.get(ALLOW_METAREDIRECTS, "true")));
        DOC_ATTR.setAllowPlugins(Boolean.parseBoolean(_userBrowserContent.get(ALLOW_PLUGINS, "true")));
        DOC_ATTR.setAllowSubframes(Boolean.parseBoolean(_userBrowserContent.get(ALLOW_SUBFRAMES, "true")));
    }

    public OptionsDialog(Frame owner)
    {
        super(owner, "Options");

        setResizable(false);
        setLayout(new BorderLayout());

        final JPanel buttonPanel = new JPanel();
        buttonPanel.setLayout(new FlowLayout(FlowLayout.RIGHT));
        buttonPanel.setBorder(new EmptyBorder(5, 5, 5, 5));

        _buttonOK = new JButton();
        _buttonOK.setText("OK");
        Dimension preferedSize;
        if (Xpcom.isMacOSX() || Xpcom.isLinux())
        {
            preferedSize = new Dimension(100, 23);
        }
        else
        {
            preferedSize = new Dimension(80, 23);
        }
        _buttonOK.setPreferredSize(preferedSize);
        buttonPanel.add(_buttonOK);
        _buttonCancel = new JButton();
        _buttonCancel.setText("Cancel");
        _buttonCancel.setPreferredSize(preferedSize);
        buttonPanel.add(_buttonCancel);

        setupUI();

        add(_contentPanel, BorderLayout.NORTH);
        add(buttonPanel, BorderLayout.SOUTH);
        add(_connectionSettingsPanel, BorderLayout.CENTER);

        setModal(true);
        getRootPane().setDefaultButton(_buttonOK);

        _buttonOK.addActionListener(new ActionListener()
        {
            public void actionPerformed(ActionEvent e)
            {
                onOK();
            }
        });

        _buttonCancel.addActionListener(new ActionListener()
        {
            public void actionPerformed(ActionEvent e)
            {
                onCancel();
            }
        });

        // call onCancel() when cross is clicked
        setDefaultCloseOperation(DO_NOTHING_ON_CLOSE);
        addWindowListener(new WindowAdapter()
        {
            public void windowClosing(WindowEvent e)
            {
                onCancel();
            }
        });

        // call onCancel() on ESCAPE
        buttonPanel.registerKeyboardAction(new ActionListener()
        {
            public void actionPerformed(ActionEvent e)
            {
                onCancel();
            }
        }, KeyStroke.getKeyStroke(KeyEvent.VK_ESCAPE, 0), JComponent.WHEN_ANCESTOR_OF_FOCUSED_COMPONENT);

        _contentPanel.registerKeyboardAction(new ActionListener()
        {
            public void actionPerformed(ActionEvent e)
            {
                onCancel();
            }
        }, KeyStroke.getKeyStroke(KeyEvent.VK_ESCAPE, 0), JComponent.WHEN_ANCESTOR_OF_FOCUSED_COMPONENT);

        _connectionSettingsPanel.registerKeyboardAction(new ActionListener()
        {
            public void actionPerformed(ActionEvent e)
            {
                onCancel();
            }
        }, KeyStroke.getKeyStroke(KeyEvent.VK_ESCAPE, 0), JComponent.WHEN_ANCESTOR_OF_FOCUSED_COMPONENT);

        _manualProxyConfigurationRadioButton.addItemListener(new ItemListener()
        {
            public void itemStateChanged(ItemEvent e)
            {
                updateManualProxyConfiguration();
            }
        });

        _automaticProxyConfigurationURLRadioButton.addItemListener(new ItemListener()
        {
            public void itemStateChanged(ItemEvent e)
            {
                updateAutomaticProxyConfiguration();
            }
        });

        _useThisProxyServerCheckBox.addItemListener(new ItemListener()
        {
            public void itemStateChanged(ItemEvent e)
            {
                updateManualProxyConfiguration();
            }
        });

        _httpProxyTextField.getDocument().addDocumentListener(new DocumentListener()
        {
            public void insertUpdate(DocumentEvent e)
            {
                updateManualProxyConfiguration();
            }

            public void removeUpdate(DocumentEvent e)
            {
                updateManualProxyConfiguration();
            }

            public void changedUpdate(DocumentEvent e)
            {
                updateManualProxyConfiguration();
            }
        });

        (_httpPortTextField.getTextField()).getDocument().addDocumentListener(new DocumentListener()
        {
            public void insertUpdate(DocumentEvent e)
            {
                updateManualProxyConfiguration();
            }

            public void removeUpdate(DocumentEvent e)
            {
                updateManualProxyConfiguration();
            }

            public void changedUpdate(DocumentEvent e)
            {
                updateManualProxyConfiguration();
            }
        });

        ProxyConfiguration proxyConfiguration = Services.getProxyConfiguration();

        final int proxyType = proxyConfiguration.getType();
        _directConnectionRadioButton.setSelected(proxyType == ProxyConfiguration.DIRECT);
        _webProxyAutoDiscoveryRadioButton.setSelected(proxyType == ProxyConfiguration.WPAD);
        _manualProxyConfigurationRadioButton.setSelected(proxyType == ProxyConfiguration.MANUAL);
        _automaticProxyConfigurationURLRadioButton.setSelected(proxyType == ProxyConfiguration.PAC);

        _useThisProxyServerCheckBox.setSelected(proxyConfiguration.getShareProxySettings());
        _httpProxyTextField.setText(proxyConfiguration.getHttpHost());
        _httpPortTextField.setText(String.valueOf(proxyConfiguration.getHttpPort()));
        _sslProxyTextField.setText(proxyConfiguration.getSslHost());
        _sslPortTextField.setText(String.valueOf(proxyConfiguration.getSslPort()));
        _ftpProxyTextField.setText(proxyConfiguration.getFtpHost());
        _ftpPortTextField.setText(String.valueOf(proxyConfiguration.getFtpPort()));
        _gopherProxyTextField.setText(proxyConfiguration.getGopherHost());
        _gopherPortTextField.setText(String.valueOf(proxyConfiguration.getGopherPort()));
        _socksProxyTextField.setText(proxyConfiguration.getSocksHost());
        _socksPortTextField.setText(String.valueOf(proxyConfiguration.getSocksPort()));
        _socksV4RadioButton.setSelected(proxyConfiguration.getSocksVersion() == 4);
        _socksV5RadioButton.setSelected(proxyConfiguration.getSocksVersion() == 5);
        _noProxyTextField.setText(proxyConfiguration.getSkipProxyFor());

        final String autoconfigUrl = proxyConfiguration.getAutoconfigUrl();
        if (autoconfigUrl != null)
        {
            _automaticProxyConfigurationURLTextField.setText(autoconfigUrl);
        }
        else
        {
            _automaticProxyConfigurationURLTextField.setText("");
        }

        updateManualProxyConfiguration();
        updateAutomaticProxyConfiguration();
    }

    private void updateAutomaticProxyConfiguration()
    {
        _automaticProxyConfigurationURLTextField.setEnabled(_automaticProxyConfigurationURLRadioButton.isSelected());
    }

    private void updateManualProxyConfiguration()
    {
        final boolean manualConfigEnabled = _manualProxyConfigurationRadioButton.isSelected();
        _httpProxyTextField.setEnabled(manualConfigEnabled);
        _httpPortTextField.setEnabled(manualConfigEnabled);
        _useThisProxyServerCheckBox.setEnabled(manualConfigEnabled);

        final boolean bothControlSelected = !_useThisProxyServerCheckBox.isSelected() && manualConfigEnabled;
        _sslProxyTextField.setEnabled(bothControlSelected);
        _ftpProxyTextField.setEnabled(bothControlSelected);
        _gopherProxyTextField.setEnabled(bothControlSelected);
        _socksProxyTextField.setEnabled(bothControlSelected);
        _sslPortTextField.setEnabled(bothControlSelected);
        _ftpPortTextField.setEnabled(bothControlSelected);
        _gopherPortTextField.setEnabled(bothControlSelected);
        _socksPortTextField.setEnabled(bothControlSelected);

        final String httpProxy = _httpProxyTextField.getText();
        final String httpPort = _httpPortTextField.getText();
        if (_useThisProxyServerCheckBox.isSelected())
        {
            _ftpProxyTextField.setText(httpProxy);
            _sslProxyTextField.setText(httpProxy);
            _gopherProxyTextField.setText(httpProxy);
            _socksProxyTextField.setText(httpProxy);
            _ftpPortTextField.setText(httpPort);
            _sslPortTextField.setText(httpPort);
            _gopherPortTextField.setText(httpPort);
            _socksPortTextField.setText(httpPort);
        }
        else
        {
            _ftpProxyTextField.setText("");
            _sslProxyTextField.setText("");
            _gopherProxyTextField.setText("");
            _socksProxyTextField.setText("");
            _ftpPortTextField.setText("0");
            _sslPortTextField.setText("0");
            _gopherPortTextField.setText("0");
            _socksPortTextField.setText("0");
        }
    }

    private void onOK()
    {
        _userBrowserContent.put(ALLOW_IMAGES, String.valueOf(_loadImagesChBox.isSelected()));
        _userBrowserContent.put(ALLOW_JAVASCRIPT, String.valueOf(_enableJavascriptChBox.isSelected()));
        _userBrowserContent.put(ALLOW_METAREDIRECTS, String.valueOf(!_blockMetaredirectsChBox.isSelected()));
        _userBrowserContent.put(ALLOW_PLUGINS, String.valueOf(_enablePluginsChBox.isSelected()));
        _userBrowserContent.put(ALLOW_SUBFRAMES, String.valueOf(!_blockSubframesChBox.isSelected()));

        ProxyConfiguration proxyConfiguration = Services.getProxyConfiguration();
        final int proxyType;
        if (_webProxyAutoDiscoveryRadioButton.isSelected())
        {
            proxyType = ProxyConfiguration.WPAD;
        }
        else if (_manualProxyConfigurationRadioButton.isSelected())
        {
            proxyType = ProxyConfiguration.MANUAL;
            if (_httpProxyTextField.getText().equals("") && _ftpProxyTextField.getText().equals("")
                    && _sslProxyTextField.getText().equals(""))
            {
                JOptionPane.showMessageDialog(this, "Please specify proxy configuration!",
                        "Manual configuration error", JOptionPane.ERROR_MESSAGE);
                return;
            }
        }
        else if (_automaticProxyConfigurationURLRadioButton.isSelected())
        {
            proxyType = ProxyConfiguration.PAC;
        }
        else
        {
            proxyType = ProxyConfiguration.DIRECT;
        }
        proxyConfiguration.setType(proxyType);

        proxyConfiguration.setShareProxySettings(_useThisProxyServerCheckBox.isSelected());
        proxyConfiguration.setHttpHost(_httpProxyTextField.getText());
        proxyConfiguration.setHttpPort(Integer.parseInt(_httpPortTextField.getText()));
        proxyConfiguration.setSslHost(_sslProxyTextField.getText());
        proxyConfiguration.setSslPort(Integer.parseInt(_sslPortTextField.getText()));
        proxyConfiguration.setFtpHost(_ftpProxyTextField.getText());
        proxyConfiguration.setFtpPort(Integer.parseInt(_ftpPortTextField.getText()));
        proxyConfiguration.setGopherHost(_gopherProxyTextField.getText());
        proxyConfiguration.setGopherPort(Integer.parseInt(_gopherPortTextField.getText()));
        proxyConfiguration.setSocksHost(_socksProxyTextField.getText());
        proxyConfiguration.setSocksPort(Integer.parseInt(_socksPortTextField.getText()));
        if (_socksV4RadioButton.isSelected())
        {
            proxyConfiguration.setSocksVersion(4);
        }
        else
        {
            proxyConfiguration.setSocksVersion(5);
        }
        proxyConfiguration.setSkipProxyFor(_noProxyTextField.getText());

        String textUrl = _automaticProxyConfigurationURLTextField.getText();
        try
        {
            new URL(textUrl);
        }
        catch (MalformedURLException e)
        {
            if (textUrl != null && !textUrl.trim().equals(""))
            {
                textUrl = "http://" + textUrl.trim() + "/";
            }
        }
        proxyConfiguration.setAutoconfigUrl(textUrl);

        DOC_ATTR.setAllowImages(_loadImagesChBox.isSelected());
        DOC_ATTR.setAllowJavascript(_enableJavascriptChBox.isSelected());
        DOC_ATTR.setAllowMetaRedirects(!_blockMetaredirectsChBox.isSelected());
        DOC_ATTR.setAllowSubframes(!_blockSubframesChBox.isSelected());
        DOC_ATTR.setAllowPlugins(_enablePluginsChBox.isSelected());

        dispose();
    }

    private void onCancel()
    {
        dispose();
    }

    private GridBagConstraints getEditGBC()
    {
        GridBagConstraints gbc = new GridBagConstraints();
        gbc.gridx = 1;
        gbc.gridy = 0;
        gbc.weightx = 0.87;
        gbc.weighty = 1.0;
        gbc.fill = GridBagConstraints.HORIZONTAL;
        return gbc;
    }

    private class CustomBorder implements Border
    {

        String title;

        CustomBorder(String title)
        {
            this.title = title;
        }

        public Insets getBorderInsets(Component c)
        {
            return new Insets(17, 0, 0, 0);
        }

        public boolean isBorderOpaque()
        {
            return false;
        }

        public void paintBorder(Component c, Graphics g, int x, int y, int width, int height)
        {
            Graphics2D g2d = (Graphics2D) g;
            if (Xpcom.isMacOSX() || Xpcom.isLinux())
            {
                g2d.setColor(Color.GRAY);
                g2d.drawLine(150, 15, c.getWidth() - 10, 15);
                g2d.setColor(Color.WHITE);
                g2d.drawLine(150, 16, c.getWidth() - 10, 16);
                g2d.setColor(Color.BLACK);
                g.drawString(title, 12, 18);
            }
            else
            {
                g2d.setColor(Color.GRAY);
                g2d.drawLine(120, 15, c.getWidth() - 10, 15);
                g2d.setColor(Color.WHITE);
                g2d.drawLine(120, 16, c.getWidth() - 10, 16);
                g2d.setColor(Color.BLACK);
                g.drawString(title, 15, 18);
            }
        }

    }

    private void setupUI()
    {
        setupContentPanel();
        setupConnectionSettingsPanel();
    }

    private void setupContentPanel()
    {
        _loadImagesChBox = new JCheckBox("Automatically load images", DOC_ATTR.isAllowImages());
        _enableJavascriptChBox = new JCheckBox("Enable javascript", DOC_ATTR.isAllowJavascript());
        _enablePluginsChBox = new JCheckBox("Enable plugins", DOC_ATTR.isAllowPlugins());
        _blockSubframesChBox = new JCheckBox("Block subframes", !DOC_ATTR.isAllowSubframes());
        _blockMetaredirectsChBox = new JCheckBox("Block meta redirects", !DOC_ATTR.isAllowMetaRedirects());

        _contentPanel = new JPanel();
        _contentPanel.setBorder(new CompoundBorder(new EmptyBorder(25, 10, 0, 15), new ContentPanelBorder("Content")));
        _contentPanel.setLayout(new BoxLayout(this, BoxLayout.X_AXIS));

        GroupLayout panelLayout = new GroupLayout(_contentPanel);
        _contentPanel.setLayout(panelLayout);
        panelLayout.setHorizontalGroup(panelLayout.createSequentialGroup()
            .add(panelLayout.createParallelGroup()
                .add(_loadImagesChBox)
                .add(_enableJavascriptChBox)
            )
            .add(5, 5, 5)
            .add(panelLayout.createParallelGroup()
                .add(_enablePluginsChBox)
                .add(_blockSubframesChBox)
            )
            .add(5, 5, 5)
            .add(_blockMetaredirectsChBox)
        );
        panelLayout.setVerticalGroup(panelLayout.createSequentialGroup()
            .add(panelLayout.createParallelGroup()
                .add(_loadImagesChBox)
                .add(_enablePluginsChBox)
                .add(_blockMetaredirectsChBox)
            )
            .add(panelLayout.createParallelGroup()
                .add(_enableJavascriptChBox)
                .add(_blockSubframesChBox)
            )
        );
    }

    private void setupConnectionSettingsPanel()
    {
        _connectionSettingsPanel = new JPanel();
        _connectionSettingsPanel.setLayout(new BoxLayout(_connectionSettingsPanel, BoxLayout.PAGE_AXIS));
        _connectionSettingsPanel.setBorder(new CompoundBorder(new EmptyBorder(11, 10, 0, 10), new CustomBorder("Connection settings")));

        // ConfigType panel
        JPanel configTypePanel = new JPanel();

        configTypePanel.setAlignmentX(Component.LEFT_ALIGNMENT);
        configTypePanel.setLayout(new BoxLayout(configTypePanel, BoxLayout.Y_AXIS));
        _directConnectionRadioButton = new JRadioButton();
        _directConnectionRadioButton.setSelected(true);
        _directConnectionRadioButton.setText("Direct connection to the Internet");
        _directConnectionRadioButton.setMnemonic('D');
        _directConnectionRadioButton.setDisplayedMnemonicIndex(0);
        configTypePanel.add(_directConnectionRadioButton);

        _webProxyAutoDiscoveryRadioButton = new JRadioButton();
        _webProxyAutoDiscoveryRadioButton.setText("Auto-detect proxy settings for this network");
        _webProxyAutoDiscoveryRadioButton.setMnemonic('A');
        _webProxyAutoDiscoveryRadioButton.setDisplayedMnemonicIndex(0);
        configTypePanel.add(_webProxyAutoDiscoveryRadioButton);

        _manualProxyConfigurationRadioButton = new JRadioButton();
        _manualProxyConfigurationRadioButton.setText("Manual proxy configuration:");
        _manualProxyConfigurationRadioButton.setMnemonic('M');
        _manualProxyConfigurationRadioButton.setDisplayedMnemonicIndex(0);
        configTypePanel.add(_manualProxyConfigurationRadioButton);

        setupManualProxyConfigurationPanel();

        JPanel automaticURLPanel = new JPanel();
        automaticURLPanel.setBorder(new EmptyBorder(0, 0, 7, 0));
        automaticURLPanel.setLayout(new GridBagLayout());
        _automaticProxyConfigurationURLRadioButton = new JRadioButton();
        _automaticProxyConfigurationURLRadioButton.setText("Automatic proxy configuration URL");
        _automaticProxyConfigurationURLRadioButton.setMnemonic('U');
        _automaticProxyConfigurationURLRadioButton.setDisplayedMnemonicIndex(30);
        _automaticProxyConfigurationURLTextField = new JTextField();
        GridBagConstraints gbc = getEditGBC();
        gbc.gridx = 0;
        gbc.gridy = 0;
        gbc.weightx = 1.0;
        gbc.weighty = 0.6;
        automaticURLPanel.add(_automaticProxyConfigurationURLRadioButton, gbc);

        gbc = getEditGBC();
        gbc.gridx = 0;
        gbc.gridy = 1;
        gbc.insets = new Insets(0, 20, 0, 0);
        automaticURLPanel.add(_automaticProxyConfigurationURLTextField, gbc);

        GroupLayout contentPanelLayout = new GroupLayout(_connectionSettingsPanel);
        _connectionSettingsPanel.setLayout(contentPanelLayout);
        contentPanelLayout.setHorizontalGroup(contentPanelLayout.createParallelGroup(GroupLayout.CENTER)
                .add(configTypePanel)
                .add(_manualProxyConfigurationPanel)
                .add(automaticURLPanel)
        );
        contentPanelLayout.setVerticalGroup(contentPanelLayout.createSequentialGroup()
                .add(configTypePanel)
                .add(_manualProxyConfigurationPanel)
                .add(automaticURLPanel)
        );

        ButtonGroup buttonGroup;
        buttonGroup = new ButtonGroup();
        buttonGroup.add(_socksV4RadioButton);
        buttonGroup.add(_socksV5RadioButton);
        buttonGroup = new ButtonGroup();
        buttonGroup.add(_directConnectionRadioButton);
        buttonGroup.add(_webProxyAutoDiscoveryRadioButton);
        buttonGroup.add(_manualProxyConfigurationRadioButton);
        buttonGroup.add(_automaticProxyConfigurationURLRadioButton);
    }

    private void setupManualProxyConfigurationPanel()
    {
        _httpProxyTextField = new JTextField();
        _httpPortTextField = new PortTextField();

        _useThisProxyServerCheckBox = new JCheckBox("Use this proxy server for all protocols");
        _useThisProxyServerCheckBox.setMnemonic('T');
        _useThisProxyServerCheckBox.setDisplayedMnemonicIndex(4);

        _sslProxyTextField = new JTextField();
        _sslPortTextField = new PortTextField();

        _ftpProxyTextField = new JTextField();
        _ftpPortTextField = new PortTextField();

        _gopherProxyTextField = new JTextField();
        _gopherPortTextField = new PortTextField();

        _socksProxyTextField = new JTextField();
        _socksPortTextField = new PortTextField();

        _socksV4RadioButton = new JRadioButton("SOCKS v4");
        _socksV4RadioButton.setMnemonic('4');
        _socksV4RadioButton.setDisplayedMnemonicIndex(7);
        _socksV5RadioButton = new JRadioButton("SOCKS v5");
        _socksV5RadioButton.setMnemonic('5');
        _socksV5RadioButton.setDisplayedMnemonicIndex(7);

        _noProxyTextField = new JTextField();

        JLabel label1 = new JLabel("HTTP Proxy:");
        JLabel label2 = new JLabel("SSL Proxy:");
        JLabel label3 = new JLabel("FTP Proxy:");
        JLabel label4 = new JLabel("Gopher Proxy:");
        JLabel label5 = new JLabel("SOCKS Proxy:");
        JLabel label6 = new JLabel("No proxy for:");

        JLabel httpPortLb = new JLabel("Port:");
        JLabel sslPortLb = new JLabel("Port:");
        JLabel ftpPortLb = new JLabel("Port:");
        JLabel gopherPortLb = new JLabel("Port:");
        JLabel socksPortLb = new JLabel("Port:");

        _manualProxyConfigurationPanel = new JPanel();
        GroupLayout panelLayout = new GroupLayout(_manualProxyConfigurationPanel);
		_manualProxyConfigurationPanel.setLayout(panelLayout);
        panelLayout.setHorizontalGroup(panelLayout.createSequentialGroup()
            .add(5, 5, 5)
            .add(panelLayout.createParallelGroup(org.jdesktop.layout.GroupLayout.TRAILING)
                .add(label1)
                .add(label2)
                .add(label3)
                .add(label4)
                .add(label5)
                .add(label6)
            )
            .add(3, 3, 3)
            .add(panelLayout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
                .add(panelLayout.createSequentialGroup()
                    .add(panelLayout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
                        .add(_httpProxyTextField)
                        .add(_useThisProxyServerCheckBox)
                        .add(_sslProxyTextField )
                        .add(_ftpProxyTextField)
                        .add(_gopherProxyTextField)
                        .add(_socksProxyTextField)
                        .add(panelLayout.createSequentialGroup()
                            .add(_socksV4RadioButton)
                            .add(5, 5, 5)
                            .add(_socksV5RadioButton)
                        )
                    )
                    .add(10, 10, 10)
                    .add(panelLayout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
                        .add(httpPortLb)
                        .add(sslPortLb)
                        .add(ftpPortLb)
                        .add(gopherPortLb)
                        .add(socksPortLb)
                    )
                    .add(3, 3, 3)
                    .add(panelLayout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
                        .add(_httpPortTextField.getTextField())
                        .add(_sslPortTextField.getTextField())
                        .add(_ftpPortTextField.getTextField())
                        .add(_gopherPortTextField.getTextField())
                        .add(_socksPortTextField.getTextField())
                    )
                )
                .add(_noProxyTextField)
            )
        );

        panelLayout.setVerticalGroup(panelLayout.createSequentialGroup()
            .add(panelLayout.createParallelGroup(org.jdesktop.layout.GroupLayout.BASELINE)
                .add(label1)
                .add(_httpProxyTextField)
                .add(httpPortLb)
                .add(_httpPortTextField.getTextField())
            )
            .add(2, 2, 2)
            .add(_useThisProxyServerCheckBox)
            .add(2, 2, 2)
            .add(panelLayout.createParallelGroup(org.jdesktop.layout.GroupLayout.BASELINE)
                .add(label2)
                .add(_sslProxyTextField)
                .add(sslPortLb)
                .add(_sslPortTextField.getTextField())
            )
            .add(2, 2, 2)
            .add(panelLayout.createParallelGroup(org.jdesktop.layout.GroupLayout.BASELINE)
                .add(label3)
                .add(_ftpProxyTextField)
                .add(ftpPortLb)
                .add(_ftpPortTextField.getTextField())
            )
            .add(2, 2, 2)
            .add(panelLayout.createParallelGroup(org.jdesktop.layout.GroupLayout.BASELINE)
                .add(label4)
                .add(_gopherProxyTextField)
                .add(gopherPortLb)
                .add(_gopherPortTextField.getTextField())
            )
            .add(2, 2, 2)
            .add(panelLayout.createParallelGroup(org.jdesktop.layout.GroupLayout.BASELINE)
                .add(label5)
                .add(_socksProxyTextField)
                .add(socksPortLb)
                .add(_socksPortTextField.getTextField())
            )
            .add(2, 2, 2)
            .add(panelLayout.createParallelGroup(org.jdesktop.layout.GroupLayout.BASELINE)
                .add(_socksV4RadioButton)
                .add(_socksV5RadioButton)
            )
            .add(2, 2, 2)
            .add(panelLayout.createParallelGroup(org.jdesktop.layout.GroupLayout.BASELINE)
                .add(label6)
                .add(_noProxyTextField)
            )
            .add(5, 5, 5)
        );
    }

    private class ContentPanelBorder implements Border
    {

        private String title;

        ContentPanelBorder(String title)
        {
            this.title = title;
        }

        public Insets getBorderInsets(Component c)
        {
            return new Insets(5, 0, 0, 0);
        }

        public boolean isBorderOpaque()
        {
            return false;
        }

        public void paintBorder(Component c, Graphics g, int x, int y, int width, int height)
        {
            Graphics2D g2d = (Graphics2D) g;
            if (Xpcom.isMacOSX() || Xpcom.isLinux())
            {
                g2d.setColor(Color.GRAY);
                g2d.drawLine(75, 16, c.getWidth() - 10, 16);
                g2d.setColor(Color.WHITE);
                g2d.drawLine(75, 17, c.getWidth() - 10, 17);
                g2d.setColor(Color.BLACK);
                g.drawString(title, 12, 20);

            }
            else
            {
                g2d.setColor(Color.GRAY);
                g2d.drawLine(65, 16, c.getWidth() - 10, 16);
                g2d.setColor(Color.WHITE);
                g2d.drawLine(65, 17, c.getWidth() - 10, 17);
                g2d.setColor(Color.BLACK);
                g.drawString(title, 15, 20);
            }
        }
    }

    private class PortTextField
    {
        private JTextField field;
        final private Color bgColorEnabled = new Color(255,255,255);
        final private Color bgColorDisable;

        public PortTextField()
        {
            if (Xpcom.isMacOSX() || Xpcom.isLinux())
            {
                bgColorDisable = new JTextField().getBackground();
            }
            else
            {
                bgColorDisable = new Color(212,208,200);
            }

            field = new JFormattedTextField(new InternationalFormatter()
            {
                private DocumentFilter filter = new IntFilter();

                protected DocumentFilter getDocumentFilter()
                {
                    return filter;
                }
            });
            field.setInputVerifier(new PortVerifier());
        }

        public String getText()
        {
            return field.getText();
        }

        public void setText(String text)
        {
            field.setText(text);
        }

        public JTextField getTextField()
        {
            return field;
        }

        public void setEnabled(boolean enabled)
        {
            if (enabled)
            {
                field.setBackground(bgColorEnabled);
            }
            else
            {
                field.setBackground(bgColorDisable);
            }
            field.setEnabled(enabled);
        }
        
    }

    private class PortVerifier extends InputVerifier
    {
        private final int stringLenth = 9;

        public boolean verify(JComponent component)
        {
            JFormattedTextField field = (JFormattedTextField) component;
            final String text = field.getText();
            if (text == null || text.equals(""))
            {
                field.setText("0");
            }
            else if (text.length() > stringLenth)
            {
                field.setText(text.substring(0, stringLenth));
            }
            
            return field.isEditValid();
        }
    }
    
    private class IntFilter extends DocumentFilter
    {
        public void insertString(FilterBypass fb, int offset, String string, AttributeSet attr) throws BadLocationException
        {
            StringBuilder builder = filtering(string);
            super.insertString(fb, offset, builder.toString(), attr);
        }

        public void replace(FilterBypass fb, int offset, int length, String string, AttributeSet attr) throws BadLocationException
        {
            if (string != null)
            {
                StringBuilder builder = filtering(string);
                string = builder.toString();
            }
            super.replace(fb, offset, length, string, attr);
        }

        private StringBuilder filtering(String string)
        {
            StringBuilder builder = new StringBuilder(string);
            for (int i = builder.length() - 1; i >= 0; i--)
            {
                int cp = builder.codePointAt(i);
                if (!Character.isDigit(cp))
                {
                    builder.deleteCharAt(i);
                    if (Character.isSupplementaryCodePoint(cp))
                    {
                        i--;
                        builder.deleteCharAt(i);
                    }
                }
            }
            return builder;
        }
    }
}
