/*
 * Copyright (c) 2000-2009 TeamDev Ltd. All rights reserved.
 * TeamDev PROPRIETARY and CONFIDENTIAL.
 * Use is subject to license terms.
 */

package com.teamdev.xpcom.mozilla.demo.components;

import java.awt.Component;
import java.awt.HeadlessException;
import java.awt.event.ActionEvent;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;
import java.io.File;

import javax.swing.AbstractAction;
import javax.swing.JDialog;
import javax.swing.JFileChooser;
import javax.swing.JOptionPane;

import com.teamdev.xpcom.Xpcom;

public class SaveJFileChooser extends JFileChooser {

    private JDialog dialog;
    private Runnable saveAction;
    private int choos;

    public SaveJFileChooser(Component parentComponent,final Runnable action, final boolean waitActionExecuting) {
        super();
        saveAction = action;
        dialog = createDialog(parentComponent);

        setDialogType(JFileChooser.SAVE_DIALOG);
        addActionListener(new AbstractAction() {
            public void actionPerformed(ActionEvent event) {
                JFileChooser fileChooser = (JFileChooser) event.getSource();
                choos = JFileChooser.APPROVE_SELECTION.equals(event.getActionCommand())
                        ? JFileChooser.APPROVE_OPTION
                        : JFileChooser.CANCEL_OPTION;
                if (JFileChooser.APPROVE_SELECTION.equals(event.getActionCommand())) {
                    File selectedFile = fileChooser.getSelectedFile();
                    if (!checkAccessForCreating(fileChooser))
                    {
                        JOptionPane.showMessageDialog(fileChooser,
                                "File " + selectedFile.getPath() + " not allowed to write. \nPlease change path or name",
                                "Notification",
                                JOptionPane.WARNING_MESSAGE);
                        dialog.setVisible(true);
                        return;
                    }
                    if (selectedFile.exists() && !selectedFile.isDirectory()) {
                        if (JOptionPane.NO_OPTION == JOptionPane.showConfirmDialog(fileChooser,
                                "File " + selectedFile.getPath() + " already exists.\nDo you want to replace it?",
                                "Notification",
                                JOptionPane.YES_NO_OPTION,
                                JOptionPane.WARNING_MESSAGE)) {
                            dialog.setVisible(true);
                            return;
                        }
                    }
                    if (waitActionExecuting) {
                        Xpcom.invokeAndWait(saveAction);
                    } else {
                        new Thread(action).start();
                    }
                }
                dialog.dispose();
            }
        });
    }

    private boolean checkAccessForCreating(JFileChooser fileChooser) {
        boolean creatingFileAvalaible;
        try {
            File fileDir = fileChooser.getFileSystemView().createNewFolder(new File(fileChooser.getCurrentDirectory().getAbsolutePath()));
            creatingFileAvalaible = fileDir.delete();
        } catch (Exception e) {
            throw new RuntimeException(e);
        }
        return creatingFileAvalaible;
    }

    @Override
    public void setVisible(boolean visible) {
        dialog.setVisible(visible);
    }

    @Override
    public JDialog createDialog(Component parentComponent) throws HeadlessException {
        JDialog jDialog = super.createDialog(parentComponent);
        jDialog.setTitle("Save As");
        // Add listener for window closing events
        jDialog.addWindowListener(new WindowAdapter() {
            public void windowClosing(WindowEvent e) {
                // Close-dialog icon was clicked
                // Hide dialog
                choos = JFileChooser.CANCEL_OPTION;
            }
        });
        return jDialog;
    }

    public int getChoose() {
        return choos;
    }
}
